function IntToHexString(iInt, iBits){
	var sResult = "";
	var iNibbles = Math.ceil(iBits / 4);
	// Remove overflowing
	iInt &= Math.pow(2, iBits+1) - 1;
	while(iNibbles > 0) {
		sResult = ("0123456789ABCDEF").charAt(iInt & 0xF) + sResult;
		iInt >>= 4;
		iNibbles--;
	}
	return sResult.bold();
}
function IntToString(iValue) {
	// Returns a "pretty" string representation of a number:
	//		number(1000000.5) == "1,000,000.5"
	if (iValue == 0) return "0";
	var sResult = "" + iValue;
	
	for (var sResult = ""; iValue > 0; iValue = Math.floor(iValue/1000)) {
		sResult = (iValue % 1000) +
			(sResult.length > 0 ? "," + sResult : "");
		if (iValue > 1000 && sResult.length % 4 < 3)
			sResult = "0" + sResult;
	}
	return sResult.bold();
} // number()

function IntToBytesString(iValue) {
	// Returns a "pretty" string representation of a number of bytes:
	//		bytes(1000000.5) == "976.57 KB"
	var aUnits = new Array(
		"bytes", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"
    );
    for (var i = 0; iValue > 1024; i++, iValue /= 1024) {}
    sResult =  IntToString(Math.ceil(iValue * 100) / 100) + // Two decimals
    	" " + aUnits[i];
	return sResult;
}// bytes()

function IntToTimeString(iValue, iGranularity) {
	// Returns a "pretty" string representation of an elapsed number of
	// milliseconds with the given granularity:
	//		time(1000000.5) == "16m 40s 1s"
	//		time(1000000.5, 1000) == "16m 40s"
	//		time(1000000.5, 60*1000) == "17m"
	var aUnits = new Array(
		new Array(1000,	unescape("%u03BCs")),
		new Array(60,	"s"),
		new Array(60,	"m"),
		new Array(24,	"h"),
		new Array(7,	"d"),
		new Array(52,	"w"),
		new Array(Infinity,	"y")
	);
	var sSign = (iValue < 0 ? "-" : "+");
	iValue = Math.abs(iValue);
	var aResult = new Array();
	if (typeof(iGranularity) == "number")
		iValue = Math.round(iValue / iGranularity) * iGranularity;
	
	for(var i = 0; iValue > 0 && i < aUnits.length; i++) {
		var iSize = aUnits[i][0],
		    sUnit = aUnits[i][1],
		    iPartialResult = iValue % iSize;
		if (iPartialResult > 0)
			aResult.unshift(Math.round(iPartialResult) + sUnit);
		iValue = Math.floor(iValue / iSize);
	}
	if (aResult.length == 0) 
		return "0";
	else
		return sSign + aResult.join(" ");
} // time()
